SystemOrganization addCategory: #ObjectUsageTracerTests!

!Behavior methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/23/2011 12:05'!
classesForlookupSelector: selector
	"Look up the given selector in my methodDictionary.
	Return the corresponding method if found.
	Otherwise chase the superclass chain and try again.
	Return nil if no method is found."
	| lookupClass classes |
	classes := OrderedCollection new.
	lookupClass := self.
	[lookupClass == nil]
		whileFalse: 
			[ classes add: lookupClass.
			(lookupClass includesSelector: selector)
				ifTrue: [^ classes].
			lookupClass := lookupClass superclass].
	^ nil! !

!Behavior methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/23/2011 12:40'!
superclassesForlookupSelector: selector
	"Look up the given selector in my methodDictionary.
	Return the corresponding method if found.
	Otherwise chase the superclass chain and try again.
	Return nil if no method is found."
	| lookupClass classes |
	classes := OrderedCollection new.
	lookupClass := self.
	[lookupClass == nil]
		whileFalse: 
			[ (lookupClass includesSelector: selector)
				ifTrue: [^ classes].
			lookupClass := lookupClass superclass.
			classes add: lookupClass.].
	^ nil! !

TestCase subclass: #ObjectUsageTracerAbstractTest
	instanceVariableNames: 'tracer'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjectUsageTracerTests'!

!ObjectUsageTracerAbstractTest methodsFor: 'helpers' stamp: 'MarianoMartinezPeck 12/27/2011 10:53'!
assertTraceReceiverClassAndMethodForReceiver: anObject slector: aSelector
	
	| classes |
	self assert: (tracer isMarked: anObject).
	"Since the method was found in the cache, the methodDicts and the superclasses may not have been used."
	self assert: (tracer isMarked: anObject class).	
	self assert: (tracer isMarked:  (anObject class lookupSelector: aSelector)).	
	
	classes := (anObject class superclassesForlookupSelector: aSelector).
	classes remove: Object ifAbsent: [].
	classes do: [:each | 
		self deny: (tracer isMarked: each).
		self deny: (tracer isMarked: each methodDict).	
		].! !

!ObjectUsageTracerAbstractTest methodsFor: 'helpers' stamp: 'MarianoMartinezPeck 12/27/2011 10:53'!
assertTraceWholeLookupHierarchyForReceiver: anObject slector: aSelector
	"It tests that duing the method lookup, all the method dictionaries and classes of the lookup are marked. The resulted compiled method should be marked as well."
	|  classes |
	
	self assert: (tracer isMarked: anObject).
	classes := (anObject class classesForlookupSelector: aSelector).
	classes do: [:each | 
		self assert: (tracer isMarked: each).
		self assert: (tracer isMarked: each methodDict).	
		].
	self assert: (tracer isMarked:  (classes last compiledMethodAt: aSelector)).	! !

!ObjectUsageTracerAbstractTest methodsFor: 'running' stamp: 'MarianoMartinezPeck 12/27/2011 10:53'!
setUp
	tracer := ObjectUsageTracer new.
	tracer unmarkAllObjects.
	tracer disableTracing.
	Object flushCache.! !

ObjectUsageTracerAbstractTest subclass: #ObjectUsageTracerBasicTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjectUsageTracerTests'!

!ObjectUsageTracerBasicTest methodsFor: 'tests - basic' stamp: 'MarianoMartinezPeck 12/22/2011 12:26'!
testManualMarkingAndUnmarking
	|  obj1 obj2 obj3 obj4 |
	
	obj1 := 'anObject'.
	tracer markObject: obj1.
	obj2 := nil.
	tracer markObject: obj2.
	obj3 := Time now.
	tracer markObject: obj3.
	obj4 := 9999999999999999999999.
	tracer markObject: obj4.

	self assert: (tracer isMarked: obj1).
	self assert: (tracer isMarked: obj1).
	self assert: (tracer isMarked: obj1).
	self assert: (tracer isMarked: obj1).
	
	tracer unmarkObject: obj1.
	tracer unmarkObject: obj2.
	tracer unmarkObject: obj3.
	tracer unmarkObject: obj4.
	
	self deny: (tracer isMarked: obj1).
	self deny: (tracer isMarked: obj1).
	self deny: (tracer isMarked: obj1).
	self deny: (tracer isMarked: obj1).
	! !

!ObjectUsageTracerBasicTest methodsFor: 'tests - basic' stamp: 'MarianoMartinezPeck 12/22/2011 11:52'!
testUnmarkAllObjects
	|  obj1 obj2 obj3 obj4 |
	
	obj1 := 'anObject'.
	tracer markObject: obj1.
	obj2 := 'anotherObject'.
	tracer markObject: obj2.
	obj3 := Time now.
	tracer markObject: obj3.
	obj4 := 9999999999999999999999.
	tracer markObject: obj4.

	tracer unmarkAllObjects.
	
	self deny: (tracer isMarked: obj1).
	self deny: (tracer isMarked: obj1).
	self deny: (tracer isMarked: obj1).
	self deny: (tracer isMarked: obj1).
	! !

ObjectUsageTracerAbstractTest subclass: #ObjectUsageTracerMethodInsideMethodTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjectUsageTracerTests'!

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesDNUMethodInsideMethod

	|  obj1 obj2 classes |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	
	obj1 fooWithDNUMethodInside: obj2.
	obj1 fooWithDNUMethodInside: obj2.
	obj1 fooWithDNUMethodInside: obj2.
	obj1 fooWithDNUMethodInside: obj2.
	
	tracer trace:  [obj1 fooWithDNUMethodInside: obj2].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #fooWithDNUMethodInside:.
	self assertTraceReceiverClassAndMethodForReceiver: obj2 slector: #doesNotUnderstand:.
! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method changing' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesDNUMethodInsideMethodChanging

	|  obj1 obj2 obj3 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	obj3 := MockClass new.
	
	obj1 fooWithDNUMethodInside: obj2.
	obj1 fooWithDNUMethodInside: obj2.
	obj1 fooWithDNUMethodInside: obj2.
	obj1 fooWithDNUMethodInside: obj2.
	
	tracer trace:  [obj1 fooWithDNUMethodInside: obj3].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #fooWithDNUMethodInside:.
	self assertTraceWholeLookupHierarchyForReceiver: obj3 slector: #doesNotUnderstand: 
	

! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method - other way' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesDNUMethodInsideMethodOtherWay

	|  obj1 obj2 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	
	tracer startTracing.
	[obj1 xxx ] on: Error do: [].
	[obj1 xxx ] on: Error do: [].
	[obj1 xxx ] on: Error do: [].
	[obj1 xxx ] on: Error do: [].
	tracer stopTracing. 
	
	tracer trace:  [obj1 fooWithDNUMethodInside: obj2].
	
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #fooWithDNUMethodInside:.
	self assertTraceWholeLookupHierarchyForReceiver: obj2 slector: #doesNotUnderstand:.
! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesMethodInsideMethod

	|  obj1 obj2 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	
	obj1 fooWithMethodInside: obj2.
	obj1 fooWithMethodInside: obj2.
	obj1 fooWithMethodInside: obj2.
	obj1 fooWithMethodInside: obj2.
	
	tracer trace:  [obj1 fooWithMethodInside: obj2].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #fooWithMethodInside:.
	self assertTraceReceiverClassAndMethodForReceiver: obj2 slector: #fooName.
	! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method changing' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesMethodInsideMethodChanging

	|  obj1 obj2 obj3 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	obj3 := MockClass new.
	
	obj1 fooWithMethodInside: obj2.
	obj1 fooWithMethodInside: obj2.
	obj1 fooWithMethodInside: obj2.
	obj1 fooWithMethodInside: obj2.
	
	tracer trace:  [obj1 fooWithMethodInside: obj3].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #fooWithMethodInside:.
	self assertTraceWholeLookupHierarchyForReceiver: obj3 slector: #fooName! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method - other way' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesMethodInsideMethodOtherWay

	|  obj1 obj2 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	
	obj2 fooName.
	obj2 fooName.
	obj2 fooName.
	obj2 fooName.
	
	tracer trace:  [obj1 fooWithMethodInside: obj2].
	
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #fooWithMethodInside:.
	self assertTraceWholeLookupHierarchyForReceiver: obj2 slector: #fooName.
! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method - other way - changing' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesMethodInsideMethodOtherWayAndChanging

	|  obj1 obj2 obj3 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	obj3 := MockClass new. 
	
	obj2 fooName.
	obj2 fooName.
	obj2 fooName.
	obj2 fooName.
	
	tracer trace:  [obj1 fooWithMethodInside: obj3].
	
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #fooWithMethodInside:.
	self assertTraceReceiverClassAndMethodForReceiver: obj3 slector: #fooName.
	! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method - other way - changing' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesPrimitiveInsideMethodOtherWayAndChanging

	|  obj1 obj2 obj3 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	obj3 := MockClass new. 
	
	obj2 primitiveMethod.
	obj2 primitiveMethod.
	obj2 primitiveMethod.
	obj2 primitiveMethod.
	
	tracer trace:  [obj1 fooWithPrimitiveMethodInside: obj3].
	
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #fooWithPrimitiveMethodInside:.
	self assertTraceReceiverClassAndMethodForReceiver: obj3 slector: #primitiveMethod.
	! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesPrimitiveMethodInsideMethod

	|  obj1 obj2 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	
	obj1 fooWithPrimitiveMethodInside: obj2.
	obj1 fooWithPrimitiveMethodInside: obj2.
	obj1 fooWithPrimitiveMethodInside: obj2.
	obj1 fooWithPrimitiveMethodInside: obj2.
	
	tracer trace:  [obj1 fooWithPrimitiveMethodInside: obj2].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #fooWithPrimitiveMethodInside:.
	self assertTraceReceiverClassAndMethodForReceiver: obj2 slector: #primitiveMethod.
	! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method changing' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesPrimitiveMethodInsideMethodChanging

	|  obj1 obj2 obj3 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	obj3 := MockClass new.
	
	obj1 fooWithPrimitiveMethodInside: obj2.
	obj1 fooWithPrimitiveMethodInside: obj2.
	obj1 fooWithPrimitiveMethodInside: obj2.
	obj1 fooWithPrimitiveMethodInside: obj2.
	
	tracer trace:  [obj1 fooWithPrimitiveMethodInside: obj3].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #fooWithPrimitiveMethodInside:.
	self assertTraceWholeLookupHierarchyForReceiver: obj3 slector: #primitiveMethod
	

! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method - other way' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesPrimitiveMethodInsideMethodOtherWay

	|  obj1 obj2 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	
	obj2 primitiveMethod.
	obj2 primitiveMethod.
	obj2 primitiveMethod.
	obj2 primitiveMethod.
	
	tracer trace:  [obj1 fooWithPrimitiveMethodInside: obj2].
	
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #fooWithPrimitiveMethodInside:.
	self assertTraceWholeLookupHierarchyForReceiver: obj2 slector: #primitiveMethod.
! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method - other way - changing' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesQuickInsideMethodOtherWayAndChanging

	|  obj1 obj2 obj3 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	obj3 := MockClass new. 
	
	obj2 quickMethod.
	obj2 quickMethod.
	obj2 quickMethod.
	obj2 quickMethod.
	
	tracer trace:  [obj1 fooWithQuickMethodInside: obj3].
	
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #fooWithQuickMethodInside:.
	self assertTraceReceiverClassAndMethodForReceiver: obj3 slector: #quickMethod.
	! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesQuickMethodInsideMethod

	|  obj1 obj2 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	
	obj1 fooWithQuickMethodInside: obj2.
	obj1 fooWithQuickMethodInside: obj2.
	obj1 fooWithQuickMethodInside: obj2.
	obj1 fooWithQuickMethodInside: obj2.
	
	tracer trace:  [obj1 fooWithQuickMethodInside: obj2].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #fooWithQuickMethodInside:.
	self assertTraceReceiverClassAndMethodForReceiver: obj2 slector: #quickMethod.
	! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method changing' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesQuickMethodInsideMethodChanging

	|  obj1 obj2 obj3 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	obj3 := MockClass new.

	obj1 fooWithQuickMethodInside: obj2.
	obj1 fooWithQuickMethodInside: obj2.
	obj1 fooWithQuickMethodInside: obj2.
	obj1 fooWithQuickMethodInside: obj2.
	
	tracer trace:  [obj1 fooWithQuickMethodInside: obj3].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #fooWithQuickMethodInside:.
	self assertTraceWholeLookupHierarchyForReceiver: obj3 slector: #quickMethod
	

! !

!ObjectUsageTracerMethodInsideMethodTest methodsFor: 'tests - method inside method - other way' stamp: 'MarianoMartinezPeck 12/27/2011 10:50'!
testTraceLookUpHierarchyMultipleTimesQuickMethodInsideMethodOtherWay

	|  obj1 obj2 |
	
	obj1 := ClassForTesting new.
	obj2 := ClassForTesting new.
	
	obj2 quickMethod.
	obj2 quickMethod.
	obj2 quickMethod.
	obj2 quickMethod.
	
	tracer trace:  [obj1 fooWithQuickMethodInside: obj2].
	
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #fooWithQuickMethodInside:.
	self assertTraceWholeLookupHierarchyForReceiver: obj2 slector: #quickMethod.
! !

ObjectUsageTracerAbstractTest subclass: #ObjectUsageTracerMethodLookupTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjectUsageTracerTests'!

!ObjectUsageTracerMethodLookupTest methodsFor: 'tests - method lookup' stamp: 'MarianoMartinezPeck 12/27/2011 10:51'!
testTraceLookUpHierarchy
	"It tests that during the method lookup, all the method dictionaries and classes of the lookup are marked. The resulted compiled method should be marked as well."
	|  obj1 |
	obj1 := ClassForTesting new.	
	tracer trace: [obj1 objectFoo ].
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #objectFoo.	! !

!ObjectUsageTracerMethodLookupTest methodsFor: 'tests - method lookup - multiple times' stamp: 'MarianoMartinezPeck 12/27/2011 10:51'!
testTraceLookUpHierarchyMultipleTimes
	"It tests that duing the method lookup, all the method dictionaries and classes of the lookup are marked. The resulted compiled method should be marked as well. In this tests, we execute the method several times, which means it will be found in the cache."
	|  obj1 classes |
	
	obj1 := ClassForTesting new.
	
	obj1 objectFoo2.
	obj1 objectFoo2.
	obj1 objectFoo2.
	obj1 objectFoo2.
	
	tracer trace: [obj1 objectFoo2 ].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #objectFoo2.
	! !

!ObjectUsageTracerMethodLookupTest methodsFor: 'tests - method lookup - multiple times' stamp: 'MarianoMartinezPeck 12/27/2011 10:51'!
testTraceLookUpHierarchyMultipleTimesPrimitiveMethod
	
	|  obj1 classes |
	
	obj1 := ClassForTesting new.
	
	obj1 primitiveMethod.
	obj1 primitiveMethod.
	obj1 primitiveMethod.
	obj1 primitiveMethod.
	
	tracer trace: [obj1 primitiveMethod ].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #primitiveMethod.
	! !

!ObjectUsageTracerMethodLookupTest methodsFor: 'tests - method lookup - multiple times' stamp: 'MarianoMartinezPeck 12/27/2011 10:51'!
testTraceLookUpHierarchyMultipleTimesQuickMethod
	
	|  obj1 classes |
	
	obj1 := ClassForTesting new.
	
	obj1 quickMethod.
	obj1 quickMethod.
	obj1 quickMethod.
	obj1 quickMethod.
	
	tracer trace: [obj1 quickMethod ].
	
	self assertTraceReceiverClassAndMethodForReceiver: obj1 slector: #quickMethod.
	! !

!ObjectUsageTracerMethodLookupTest methodsFor: 'tests - method lookup - multiple times' stamp: 'MarianoMartinezPeck 12/27/2011 10:51'!
testTraceLookUpHierarchyMultipleTimesWithDNU	
	"It tests that duing the method lookup, all the method dictionaries and classes of the lookup are marked. The resulted compiled method should be marked as well."
	|  obj1 classes |
	
	obj1 := ClassForTesting new.
	
	tracer startTracing.
	[obj1 xxx ] on: Error do: [].
	[obj1 xxx ] on: Error do: [].
	[obj1 xxx ] on: Error do: [].
	[obj1 xxx ] on: Error do: [].
	tracer stopTracing. 
	
	self assert: (tracer isMarked: obj1).
	classes := obj1 class allSuperclasses.
	classes do: [:each | 
		self assert: (tracer isMarked: each).
		self assert: (tracer isMarked: each methodDict).	
		].
	
	classes := (obj1 class classesForlookupSelector: #doesNotUnderstand:).
	classes do: [:each | 
		self assert: (tracer isMarked: each).
		self assert: (tracer isMarked: each methodDict).	
		].
	self assert: (tracer isMarked:  (classes last compiledMethodAt: #doesNotUnderstand:)).	
	! !

!ObjectUsageTracerMethodLookupTest methodsFor: 'tests - method lookup' stamp: 'MarianoMartinezPeck 12/27/2011 10:51'!
testTraceLookUpHierarchyPrimitiveMethod
	"This is as #testTraceLookUpHierarchy but with the difference that the method has a primitive.  A special bytecode is associated for those methods. "
	| obj1 |
	obj1 := ClassForTesting new.
	tracer trace: [ obj1 primitiveMethod ].
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #primitiveMethod! !

!ObjectUsageTracerMethodLookupTest methodsFor: 'tests - method lookup' stamp: 'MarianoMartinezPeck 12/27/2011 10:51'!
testTraceLookUpHierarchyQuickMethod
	"This is as #testTraceLookUpHierarchy but with the difference that the method is quick. A quick method is one that ONLY does  a return self or return true, return false, etc. A special bytecode is associated for those methods. "
	| obj1 |
	obj1 := ClassForTesting new.
	tracer trace: [ obj1 quickMethod ].
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #quickMethod! !

!ObjectUsageTracerMethodLookupTest methodsFor: 'tests - method lookup' stamp: 'MarianoMartinezPeck 12/27/2011 10:51'!
testTraceLookUpHierarchyWithDNU
	"It tests that duing the method lookup, all the method dictionaries and classes of the lookup are marked. The resulted compiled method should be marked as well."
	|  obj1 classes |
	
	obj1 := ClassForTesting new.
	
	tracer startTracing.
	 [obj1 xxx ] on: Error do: [].
	tracer stopTracing. 
	
	self assert: (tracer isMarked: obj1).
	classes := obj1 class allSuperclasses.
	classes do: [:each | 
		self assert: (tracer isMarked: each).
		self assert: (tracer isMarked: each methodDict).	
		].
	
	classes := (obj1 class classesForlookupSelector: #doesNotUnderstand:).
	classes do: [:each | 
		self assert: (tracer isMarked: each).
		self assert: (tracer isMarked: each methodDict).	
		].
	self assert: (tracer isMarked:  (classes last compiledMethodAt: #doesNotUnderstand:)).	
	! !

ObjectUsageTracerAbstractTest subclass: #ObjectUsageTracerSpecialSelectorsTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjectUsageTracerTests'!

!ObjectUsageTracerSpecialSelectorsTest methodsFor: 'tests ' stamp: 'MarianoMartinezPeck 12/27/2011 10:59'!
testSpecialSelectorClass
	"It tests that during the method lookup, all the method dictionaries and classes of the lookup are marked. The resulted compiled method should be marked as well."
	|  obj1 |
	obj1 := ClassForTesting.	
	tracer trace: [obj1 class ].
	self assertTraceWholeLookupHierarchyForReceiver: obj1 slector: #class.	! !

Object subclass: #ClassForTestingSuperclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjectUsageTracerTests'!

ClassForTestingSuperclass subclass: #ClassForTesting
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjectUsageTracerTests'!

Object subclass: #MockClassSuperclass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjectUsageTracerTests'!

MockClassSuperclass subclass: #MockClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'ObjectUsageTracerTests'!

!MockClass methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 12/26/2011 16:50'!
fooName
	"Just do something different than the implementation of Object"
	Date name.! !

!MockClass methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 12/26/2011 16:54'!
primitiveMethod
	"this is a mock of the shallow copy"
	<primitive: 148>! !

!MockClass methodsFor: 'as yet unclassified' stamp: 'MarianoMartinezPeck 12/26/2011 16:56'!
quickMethod
	^ false! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/23/2011 11:39'!
foo: anObject
	anObject name! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/26/2011 16:48'!
fooName
	"do something just not to be quick"
	TestCase name.! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/26/2011 15:19'!
fooWithDNUMethodInside: anObject
	[anObject xxx] on: Error do: [].! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/26/2011 16:52'!
fooWithMethodInside: anObject
	anObject fooName! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/26/2011 15:16'!
fooWithPrimitiveMethodInside: anObject
	anObject primitiveMethod! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/26/2011 17:00'!
fooWithQuickMethodInside: anObject
	anObject quickMethod! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/26/2011 13:35'!
objectFoo
	"just do something so that not bo be quick"
	Transcript name.
	! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/26/2011 13:35'!
objectFoo2
	"just do something so that not bo be quick"
	Transcript name.
	! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/26/2011 14:39'!
primitiveMethod
	"this is a mock of the shallow copy"
	<primitive: 148>! !

!Object methodsFor: '*ObjectUsageTracerTests' stamp: 'MarianoMartinezPeck 12/26/2011 13:37'!
quickMethod
	^ true! !
